      subroutine build (n,n3,seed,alpha,srtc,srtic,
     +               a,b,c,objv, srnext,srprev,
     +               iprev,jprev,kprev,inext,jnext,
     +               knext,posind)

c     ------------------------------------------------------------------
c     build:  GRASP construction phase. Builds a feasible solution.
c     
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - ap3 dimension
c     n3     - n * n * n
c     seed   - random number generator seed
c     alpha  - construction phase parameter
c     objv   - cost of initial solution (returned)
c     ------------------------------------------------------------------
      implicit none
      integer n,n3,seed,objv
      real    alpha
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     srtc   - sorted costs
c     srtic  - sorted costs (indices)
c     a      - permutation array
c     b      - permutation array
c     srnext  - next pointers to the sorted cost list
c     srprev  - previous pointers to the sorted cost list
c     inext   - next pointers to the list of valid i's
c     iprev   - previous pointers to the list of valid i's
c     jnext   - next pointers to the list of valid ai's
c     jprev   - previous pointers to the list of valid ai's
c     knext   - next pointers to the list of valid bi's
c     kprev   - previous pointers to the list of valid bi's
c     posind  - index of original array elements in the sorted  array
c
c     ------------------------------------------------------------------
      integer srtic(n3),a(n),b(n)
      integer srtc(n3),c(n3)
      integer srnext(n3),srprev(n3)
      integer inext(n), jnext(n), knext(n)
      integer iprev(n),jprev(n),kprev(n)
      integer posind(n3)
c     ------------------------------------------------------------------
c     Local scalars and arrays:
c     
c     
c     nselct  - index of randomly selected element
c     i       - loop index/size of list of chosen indices
c     j       - loop index
c     randp   - random number generator function
c     xrand   - dummy probability
c     ind     - index
c     nrcl    - size of rcl
c     ifirst  - pointer to the first element of valid i's list
c     ilast   - pointer to the last element of valid i's list
c     jfirst  - pointer to the first element of valid a's list
c     jlast   - pointer to the last element of valid a's list
c     kfirst  - pointer to the first element of valid b's list
c     klast   - pointer to the last element of valid b's list
c     ------------------------------------------------------------------
      integer nselct,i,j,ind,current
      real    randp,xrand
      integer minc, maxc, first, last, cutoff
      integer nrcl
      integer ifirst,ilast,jfirst,jlast,kfirst,klast
c     ------------------------------------------------------------------
c     Initialize pointers to the list of sorted costs.
c     ------------------------------------------------------------------

       do 10 j=1,n3
         srnext(j) = j+1     
         srprev(j) = j-1     
 10    continue

       srnext(n3) = 0

       first=1
       last=n3

c     ------------------------------------------------------------------
c     Initialize pointers to the lists of valid indexes.
c     ------------------------------------------------------------------

       do 20 j=1,n
         inext(j)  = j+1
         jnext(j)  = j+1
         knext(j)  = j+1 
         iprev(j)  = j-1
         jprev(j)  = j-1
         kprev(j)  = j-1
 20   continue
       
       inext(n)  = 0
       jnext(n)  = 0
       knext(n)  = 0

       ifirst=1
       jfirst=1
       kfirst=1
       ilast=n     
       jlast=n    
       klast=n
       
c     ------------------------------------------------------------------
c     Construct solution. Iterate n-1 times.
c     ------------------------------------------------------------------
      
      objv=0
      do 70 i=1,(n-1)

c     ------------------------------------------------------------------
c     Get size of RCL.
c     ------------------------------------------------------------------
 
          minc=srtc(first)
          maxc=srtc(last)
          cutoff=minc+alpha*(maxc-minc)
          nrcl=0

          current = first
          
30        if (current.ne.0) then
             if (srtc(current).le. cutoff) then
		  nrcl=nrcl+1
              else
		  goto 40
	      endif

             current = srnext(current)
             goto 30
          endif 


c     ------------------------------------------------------------------
c     Select element, at random, from the RCL.
c     ------------------------------------------------------------------

40        xrand=randp(seed)
	  nselct=1+nrcl*(dble(seed)/dble(2147483647))


c     ------------------------------------------------------------------
c     Make selected assignments to permutation arrays.
c     ------------------------------------------------------------------

          j=0
          current = first
50        if (current.ne.0) then
             j=j+1
             if (j.eq.nselct) goto 60
             current = srnext(current)
             goto 50
          endif 

60        objv=objv+srtc(current)    
          ind=1+(srtic(current)-1)/(n*n)
          a(ind)=1+(srtic(current)-1)/n-n*(ind-1)
          b(ind)=srtic(current)-n*n*(ind-1)-n*(a(ind)-1)

          call updrcl(n,n3,srtic,ind,a(ind),b(ind),
     +               srnext,srprev,first, last,
     +               iprev,jprev,kprev,inext,jnext,knext,
     +               ifirst,ilast,jfirst,jlast,kfirst,klast,posind)

70      continue

c     ------------------------------------------------------------------
c     Make last, trivial, assignments to permutation arrays.
c     ------------------------------------------------------------------

      objv=objv+srtc(first)    
      ind=1+(srtic(first)-1)/(n*n)
      a(ind)=1+(srtic(first)-1)/n-n*(ind-1)
      b(ind)=srtic(first)-n*n*(ind-1)-n*(a(ind)-1)
   
      return
      end

