      subroutine gtpath(n,n3,seed,dffact,obj,a,b,c,nelt,maxelt,
     +            aelt,belt,celt,objb,objw,indb,
     +            indw,npool,trand,trgtpl,posopt,elitex)   

      implicit none

c     ------------------------------------------------------------------
c     gtpath: Prepares source and target solution and calls Path 
c     relinking.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     obj    - object function value
c     seed   - random number generator seed
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     objb   - cost of the best solution in the pool
c     objw   - cost of the worst solution in the pool
c     indb   - position of the best solution in the pool
c     indw   - position of the worst solution in the pool
c     npool  - index of  permutation in the pool
c     posopt - flag that indicates if path relinking is been done
c              during a pos-optimization phase
c     elitex - flag that indicates if the pool of elite has changed
c     trand  - indicates if target solution is obtained randomly
c     trgtpl - when true indicates that path relinking target solution
c              is a solution of the pool  and the source is the solution
c              obtained by the previous GRASP iteration.
c              when false the oposite direction is considered.
c  
c     ------------------------------------------------------------------
      integer n,n3,obj,seed,nelt,maxelt,npool
      integer objb,objw,indb,indw
      real*4  dffact
      logical posopt, elitex,trand,trgtpl
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c     - cost matrix
c     a     - permutation array
c     b     - permutation array
c     aelt  - a permutations array in the pool of elite solutions
c     belt  - b permutations array in the pool of elite solutions
c     celt  - array of elite solution costs
c     
c     ------------------------------------------------------------------
      integer a(n),b(n),c(n3)
      integer aelt(n*maxelt),belt(n*maxelt),celt(n*maxelt)
c     ------------------------------------------------------------------
c     Local scalars:
c
c     randp  - random number generator function
c     xrand  - dummy probability
c     cpool  - object function value of chosen pool solution
c
c     ------------------------------------------------------------------
      real     randp,xrand
      integer  cpool
c     ------------------------------------------------------------------
c     Local arrays:
c
c     apool  - array of  permutation a in the pool
c     bpool  - array of  permutation b in the pool
c     ------------------------------------------------------------------
      integer apool(n),bpool(n)
c     ------------------------------------------------------------------

     if ((nelt.eq.maxelt).or.(posopt)) then

       if (trand) then
c     ------------------------------------------------------------------
c     Get randonly the index of the solution in the pool
c     ------------------------------------------------------------------

           xrand=randp(seed)
           npool=1+seed/(2147483647/nelt)
        endif

c     ------------------------------------------------------------------
c     Copy pool solution into auxiliary arrays apool and bpool
c     ------------------------------------------------------------------

        call getrgt(n,maxelt,npool,aelt,belt,celt,apool,bpool,cpool)
         

c     ------------------------------------------------------------------
c     Call path relinking according to the given direction:
c       trgtpl is true: solution in the pool is the target
c       trgtgp is true: last GRASP solution is the target
c     ------------------------------------------------------------------

c        ---------------------------------------------------------------
c        Pool solution is the target.
c        ---------------------------------------------------------------

        if (trgtpl)  then
c     ------------------------------------------------------------------
c     Copy solution a,b,objv into auxiliary arrays
c     ------------------------------------------------------------------

           call savsol(n,a,b,objv,stora,storb,sobj)

           do  10 pos=1,rpttns
          
              call  path(n,n3,seed,dffact,a,b,obj,apool,bpool,
     +        c,nelt,maxelt,aelt,belt,celt,objb,objw,indb,
     +         indw,elitex)

              call savsol(n,stora,storb,sobj,a,b,objv)

10         continue

        endif


c        ---------------------------------------------------------------
c        GRASP solution is the target.
c        ---------------------------------------------------------------
        if (trgtgp)  then
           do  20 pos=1,rpttns
             call  path(n,n3,seed,dffact,apool,bpool,cpool,
     +        a,b,c,nelt,maxelt,aelt,belt,celt,objb,objw,indb,
     +        indw,elitex)
20         continue
         endif

       else

         call insrpl(n,a,b,nelt,maxelt,aelt,belt,
     +                   celt,objb,objw,indb,indw,objv)
         elitex = .true.


       endif

       return
       end
  
