      logical function diffpl(n,a,b,dffact,nelt,maxelt,aelt,belt)
      implicit none
c     ------------------------------------------------------------------
c     diffpl: Returns true if solution a,b is different from all 
c     solutions in the pool of elite solutions. Otherwise, returns false.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     ------------------------------------------------------------------
      integer n,nelt,maxelt
      real*4  dffact
c     ------------------------------------------------------------------
c     Passed arrays:
c     a      - permutation array
c     b      - permutation array
c     aelt   - a permutation arrays in the pool of elite solutions
c     belt   - b permutation arrays in the pool of elite solutions
c     ------------------------------------------------------------------
      integer a(n), b(n)
      integer aelt(n*maxelt), belt(n*maxelt)
c     ------------------------------------------------------------------
c     Local scalars:
c     i      - loop index
c     j      - loop index
c     delta  - distance between a,b and aelt,belt in the pool
c     isdif  - indicates if permutation a,b is different from
c                   all permutations in the pool
c     ------------------------------------------------------------------
      integer i,j
      integer delta
      logical isdif
c     ------------------------------------------------------------------
      isdif =.true.
       
       do 20 i=1,nelt
          delta = 0
          do 10 j=1,n
             if (a(j).ne.aelt((i-1)*n+j)) delta = delta +1
             if (b(j).ne.belt((i-1)*n+j)) delta = delta +1
10        continue
          if (delta.lt.(dffact*2*n)) then
              isdif =.false.
              goto 30
          endif
20    continue

30    diffpl = isdif
      return
      end

  
