      subroutine g3aps(in,n,n3,maxn3,maxn,infty,seedit,seedp,iter,
     +                  maxitr,
     +                  look4,c,nelt,maxelt,aelt,belt,celt,
     +                  objb,objw,indb,indw,iterb,dffact,prfrq,
     +                  trand,trgtpl,trgtgp,all,dopath,ittype,
     +                  apool,bpool,stora,storb,
     +                  adiff,bdiff,aprev,bprev,anext,bnext,
     +                  alist,blist,backa,backb,backobj,srtc,srtic,                            
     +                  a,b,srnext,srprev,
     +                  iprev,jprev,kprev,inext,jnext,knext,posind)

      implicit none

c     ------------------------------------------------------------------
c     Passed input scalars:
c     
c     n      - dimension of ap3 problem
c     maxn3  - maxn*maxn*maxn
c     infty  - a large integer
c     maxitr - maximum number of GRASP iterations
c     look4  - if solution of cost look4 or less is found g3aps 
c                  returns that solution
c     maxelt - maximun number of elements in the pool of
c                  elite solutions
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     prfrq  - frequency of PR post-opt in GRASP
c     trand  - indicates if target solution is obtained randomly
c     trgtpl - indicates if path relinking is to be done torward 
c              a solution in the pool 
c     trgtgp - indicates if path relinking is to be done torward 
c              the solution obtained by the last GRASP iteration
c     all    - indicates if path relinking will be applied to all
c              elite solutions
c     dopath - indicates if path relinking will be done
c     ittype - type of iteration summary: = 0, silent
c                                         = 1, print improving solutions
c                                         = 2, print all iterations
c
c     ------------------------------------------------------------------
      integer   in,n,maxn3,look4,maxn,infty
      real      randp
      integer   maxitr,ittype
      integer   maxelt,prfrq
      real*4    dffact
      logical   trand,trgtpl,trgtgp,all,dopath,elitex
c     ------------------------------------------------------------------
c     Passed input/output scalar:
c   
c     nelt      - number of elements in the pool of elite solutions  
c     seedg     - seed for GRASP
c     seedp     - seed for path relinking
c     objb      - cost of best assignment in the pool
c     objw      - cost of worst assignment in the pool
c     indb      - index of best assignment in the pool
c     indw      - index of worst assignment in the pool
c     iterb     - iteration that found the  best solution
c
c     ------------------------------------------------------------------
      integer   seedit,seedg,seedp
      integer   objb,objw,indb,indw,nelt,iterb
c     ------------------------------------------------------------------
c     Passed output scalars: 
c     
c     iter      - number of GRASP iterations taken
c     
c     ------------------------------------------------------------------
      integer  iter
c     ------------------------------------------------------------------
c     Passed input array:
c     
c     c 	  - cost matrix stored as a one-dimensional array,
c     row by row (dim = n3 = n*n*n)	
c     
c     ------------------------------------------------------------------
      integer	   c(maxn3)
c     ------------------------------------------------------------------  
c     Passed work arrays:
c     
c     aelt    - a permutations array in the pool of elite solutions
c     belt    - b permutations array in the pool of elite solutions
c     celt    - array of elite solution costs
c     apool   - array of  permutation a in the pool
c     bpool   - array of  permutation b in the pool
c     stora   - auxiliary array for permutation a
c     storb   - auxiliary array for permutation b
c     adiff  - array of symetric differences of a
c     aprev  - pointers to previous elements in differences list of a
c     anext  - pointers to next elements in differences list of a
c     alist  - differences linked list of a
c     bdiff  - array of symetric differences of b
c     bprev  - pointers to previous elements in differences list of b
c     bnext  - pointers to next elements in differences list ob b
c     blist  - differences linked list of b 
c     btrgt  - array of target permutation b in the pool
c     a	      - permutation vector (dim = n)
c     b	      - permutation vector (dim = n)
c     srtc    - sorted cost values 
c     srtic   - sorted cost values (indices)
c     srnext  - next pointers to the sorted cost list
c     srprev  - previous pointers to the sorted cost list
c     inext   - next pointers to the list of valid first assignments
c     iprev   - previous pointers to the list of valid first assignments
c     jnext   - next pointers to the list of valid second assignments
c     jprev   - previous pointers to the list of valid second assignments
c     knext   - next pointers to the list of valid third assignments
c     kprev   - previous pointers to the list of valid third assignments
c     posind  - index of original array elements in the sorted  array
c
c     ------------------------------------------------------------------
      integer adiff(maxn),aprev(maxn),anext(maxn),alist(maxn)
      integer bdiff(maxn),bprev(maxn),bnext(maxn),blist(maxn) 
      integer backa(maxn),backb(maxn),backobj
      integer aelt(n*maxelt),belt(n*maxelt),celt(maxelt)
      integer apool(maxn),bpool(maxn)
      integer stora(maxn),storb(maxn) 
      integer a(maxn),b(maxn),srtc(maxn3),srtic(maxn3)
      integer srnext(n3),srprev(n3)
      integer inext(n), jnext(n), knext(n)
      integer iprev(n),jprev(n),kprev(n)
      integer posind(n3)
     
c     ------------------------------------------------------------------
c     Local scalars:
c
c     objv   - objective function value
c     n3     - n*n*n
c     alpha  - construction phase parameter
c     pos    - index of solution in the pool
c     sumelt - sum of elite costs
c     i      - loop index
c     seed0  - seed at the beginning of iteration
c     ------------------------------------------------------------------
      integer   n3,objv,pos,sumelt,i,seed0
      real      alpha,xrand,randp
c     ------------------------------------------------------------------

      objb=infty      
      n3=n*n*n

      call srtcst(n3,c,srtc,srtic,posind)
     

c     ------------------------------------------------------------------
c     GRASP iterations
c     ------------------------------------------------------------------
      do 20 iter=1,maxitr
         seed0 = seedit
         xrand=randp(seedit)
         seedg = seedit
         alpha=randp(seedg)

c        ---------------------------------------------------------------
c        Construction phase
c        ---------------------------------------------------------------

        
c         print *, 'Custos Ordenados:'
c         print *, (srtc(i),i=1,n3)
c         print *, 'Indices:'
c         print *, (srtic(i),i=1,n3)
c         print *, 'Custos:'
c         print *, (c(i),i=1,n3)
c         stop
 
         call  build(n,n3,seedg,alpha,srtc,srtic,
     +               a,b,c,objv, srnext,srprev,
     +               iprev,jprev,kprev,inext,jnext,knext,posind)

         call debcost (maxelt,nelt,n,n3,c,a,b,objv,aelt,belt,celt,
     +                 seed0,iter) 

c        ---------------------------------------------------------------
c        Local search phase
c        ---------------------------------------------------------------

         call local(n,n3,objv,a,b,c)

c        ---------------------------------------------------------------
c        Path relinking
c        ---------------------------------------------------------------

	 if (dopath) then
              call gtpath(n,n3,seedp,dffact,objv,a,b,c,nelt,maxelt,
     +            aelt,belt,celt,objb,objw,indb,indw,
     +            trand,trgtpl,trgtgp,all,elitex,iter,iterb,ittype,
     +            apool,bpool,stora,storb,
     +            adiff,bdiff,aprev,bprev,anext,bnext,
     +            alist,blist,backa,backb,backobj) 
                            

c         --------------------------------------------------------------
c         Do intermediate path-relinking post opt
c         --------------------------------------------------------------
	     if ((mod(iter,prfrq).eq.0).and.(nelt.eq.maxelt)
     +                                             .and.elitex)then
                call psopt (n,n3,seedp,dffact,c,nelt,maxelt,
     +             aelt,belt,celt,objb,objw,
     +             indb,indw,iter,iterb,ittype,
     +             adiff,bdiff,aprev,bprev,anext,bnext,
     +             alist,blist,backa,backb,backobj)  
          endif
       else
	  if (objv .lt. objb) then
               iterb = iter
	       objb=objv
	       objw=objv
	       nelt=1
	       celt(1)=objb
	       indb=1
	       indw=1
	       do 888 i=1,n
                    aelt(i)=a(i)
                    belt(i)=b(i)
888            continue
	  endif
       endif
 

c        ------------------------------------------------------------
c           If cost of assignment is as good as wished, return.
c        ------------------------------------------------------------

          if (objb .le. look4) return



      if (ittype.eq.2) then

         sumelt =0
              do 27 i=1,nelt
              sumelt = sumelt + celt(i)
27       continue


         write (6,127) iter,objv,objw,float(sumelt)/nelt,objb
127      format (' iter: ',i7,' objv: ',i7,' worst: ',i7,
     +            ' avg: ', f10.2,' best: ', i7)
         call myflsh
      endif

 
20    continue
      iter=maxitr

      return
      end
    
