      subroutine gtpath(n,n3,seed,dffact,obj,a,b,c,nelt,maxelt,
     +         aelt,belt,celt,objb,objw,indb,indw,trand,
     +         trgtpl,trgtgp,all,elitex,iter,iterb,ittype,
     +         apool,bpool,stora,storb,
     +         adiff,bdiff,aprev,bprev,anext,bnext,
     +         alist,blist,backa,backb,backobj)  

      implicit none
c     ------------------------------------------------------------------
c     gtpath: Prepares source and target solution and calls Path 
c     relinking.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     obj    - object function value
c     seed   - random number generator seed
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     elitex - flag that indicates if the pool of elite has changed
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     objb   - cost of the best solution in the pool
c     objw   - cost of the worst solution in the pool
c     indb   - position of the best solution in the pool
c     indw   - position of the worst solution in the pool
c     trand  - indicates if target solution is obtained randomly
c     trgtpl - indicates if path relinking is to be done torward 
c              a solution in the pool 
c     trgtgp - indicates if path relinking is to be done torward 
c              the solution obtained by the last GRASP iteration
c     all    - indicates if path relinking will be applied to all
c              elite solutions
c     iterb     - iteration that found the  best solution 
c     iter      - number of GRASP iterations taken
c     ittype - type of iteration summary: = 0, silent
c                                         = 1, print improving solutions
c                                         = 2, print all iterations
c  
c     ------------------------------------------------------------------
      integer n,n3,obj,seed,nelt,maxelt,ittype
      integer objb,objw,indb,indw,iter,iterb
      real*4  dffact
      logical trand,trgtpl,trgtgp,all
      logical elitex
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c     - cost matrix
c     a     - permutation array
c     b     - permutation array
c     aelt  - a permutations array in the pool of elite solutions
c     belt  - b permutations array in the pool of elite solutions
c     celt  - array of elite solution costs
c     apool  - array of  permutation a in the pool
c     bpool  - array of  permutation b in the pool
c     stora  - auxiliary array for permutation a
c     storb  - auxiliary array for permutation b
c     adiff  - array of symetric differences of a
c     aprev  - pointers to previous elements in differences list of a
c     anext  - pointers to next elements in differences list of a
c     alist  - differences linked list of a
c     bdiff  - array of symetric differences of b
c     bprev  - pointers to previous elements in differences list of b
c     bnext  - pointers to next elements in differences list ob b
c     blist  - differences linked list of b 
c     btrgt  - array of target permutation b in the pool
c
c     ------------------------------------------------------------------
      integer adiff(n),aprev(n),anext(n),alist(n)
      integer bdiff(n),bprev(n),bnext(n),blist(n) 
      integer backa(n),backb(n),backobj
      integer apool(n),bpool(n)
      integer stora(n),storb(n) 
      integer a(n),b(n),c(n3)
      integer aelt(n*maxelt),belt(n*maxelt),celt(n*maxelt)
c     ------------------------------------------------------------------
c     Local scalars:
c
c     randp  - random number generator function
c     xrand  - dummy probability
c     cpool  - object function value of chosen pool solution
c     sobj   - auxiliary variable for object function
c     rpttn  - number of repetitions path relinking procedure in one
c              direction
c     npool  - index of  permutation in the pool
c     pos    - loop index
c
c     ------------------------------------------------------------------
      real     randp,xrand
      integer  cpool,sobj,rpttn,pos,npool
c     ------------------------------------------------------------------

      if (nelt.eq.maxelt) then

c     ------------------------------------------------------------------
c     Set the number of times path relinking will be applied in
c     each direction.
c     ------------------------------------------------------------------

       if (all) then
         rpttn = nelt
       else
         rpttn = 1
       endif

c     ------------------------------------------------------------------
c     Get index of solution in the pool randonly.
c     ------------------------------------------------------------------

       if (trand) then
           xrand=randp(seed)
           npool=1+seed/(2147483647/nelt)
           rpttn = 1
        endif

c     ------------------------------------------------------------------
c      Copy solution a,b,objv into auxiliary arrays.
c     ------------------------------------------------------------------

        call savsol(n,a,b,obj,stora,storb,sobj)  
     
c     ------------------------------------------------------------------
c     Call path relinking according to the given direction and number
c     of repetitions.
c     ------------------------------------------------------------------

        do 10 pos=1,rpttn
          
           if (.not.trand) npool = pos

c    ---------------------------------------------------------------
c      GRASP solution is the target.
c    ---------------------------------------------------------------

            if (trgtgp)  then

c     ------------------------------------------------------------------
c     Copy pool solution into auxiliary arrays apool and bpool
c     ------------------------------------------------------------------

               call getrgt(n,maxelt,npool,aelt,belt,celt,
     +                   apool,bpool,cpool)

c     ------------------------------------------------------------------
c     Apply path relinking to chosen solutions
c     ------------------------------------------------------------------ 
              
               call  path(n,n3,seed,dffact,apool,bpool,cpool,
     +         a,b,c,nelt,maxelt,aelt,belt,celt,objb,objw,indb,
     +         indw,elitex,iter,iterb,ittype,
     +         adiff,bdiff,aprev,bprev,anext,bnext,
     +         alist,blist,backa,backb,backobj)

   
            endif

c     ---------------------------------------------------------------
c      Pool solution is the target.
c     ---------------------------------------------------------------

            if (trgtpl)  then   
    
                 call getrgt(n,maxelt,npool,
     +                          aelt,belt,celt,apool,bpool,cpool)
             
                 call  path(n,n3,seed,dffact,a,b,obj,apool,bpool,
     +                c,nelt,maxelt,aelt,belt,celt,objb,objw,indb,
     +                indw,elitex,iter,iterb,ittype,
     +                adiff,bdiff,aprev,bprev,anext,bnext,
     +                alist,blist,backa,backb,backobj)

                 call savsol(n,stora,storb,sobj,a,b,obj)
        
            endif

10       continue

       else
         if (obj.lt.objb) iterb = iter 
         call insrpl(n,a,b,nelt,maxelt,aelt,belt,
     +                   celt,objb,objw,indb,indw,obj)
         elitex = .true.
       endif

       return
       end
  
