      subroutine local(n,n3,objv,a,b,c)

      implicit none
c     ------------------------------------------------------------------
c     local: Local 2-exchange on permutation array a. 
c     Returns improved permutation arrays a and b
c     as well as integer objv.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     objv   - object function value
c     
c     ------------------------------------------------------------------
      integer n,n3,objv
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c      - cost matrix
c     a      - permutation array
c     b      - permutation array
c     
c     ------------------------------------------------------------------
      integer a(n),b(n),c(n3)
c     ------------------------------------------------------------------
c     Local scalars:
c     
c     i      - permutation index/counter
c     j      - permutation index/counter
c     temp   - temp scalar used to swap a(k1) and a(k2)
c     gain   - gain from switch
c     lower  - objective function improvement
c     anotb  - tells which permutation is updated 
c              if anotb=true then a is regarded
c              if anotb=false then b is regarded
c     quit   - outer loop quitflag
c     stop   - inner loop stopflag
c     
c     ------------------------------------------------------------------
      integer i,j,temp,gain 
      logical quit,stop,lower,anotb
c     ------------------------------------------------------------------
c     Search through the neighborhood. Initialize quitflag as false.
c     ------------------------------------------------------------------
      quit=.false.
c     ------------------------------------------------------------------
c     If quit end search.
c     ------------------------------------------------------------------
 10   if (.not.quit) then 
         stop=.false.
c     ------------------------------------------------------------------
c     Produce a 2-exchange permutation of i and j. Apply it 
c     to a and b thus creating new permutations.
c     Check if new permutations are better than old ones.
c     If so, stop the search, update the solution, i.e. the permutation,
c     and search the new neighborhood instead.
c     ------------------------------------------------------------------
         i=1
 20      if ((.not.stop).and.(i.le.(n-1))) then
            j=i+1
 30         if ((.not.stop).and.(j.le.n)) then
               anotb=.true.
               call costdf(n,n3,a,b,i,j,lower,c,anotb,gain) 
               if (lower) then
                  objv=objv-gain
                  temp=a(i)
                  a(i)=a(j)
                  a(j)=temp
                  stop=.true.
               else
                  anotb=.false.
                  call costdf(n,n3,a,b,i,j,lower,c,anotb,gain) 
                  if (lower) then
                     objv=objv-gain		    
                     temp=b(i)
                     b(i)=b(j)
                     b(j)=temp
                     stop=.true.
                  end if
                  if ((.not.stop).and.((i.eq.(n-1)).and.(j.eq.n))) then
                  quit=.true.
                  end if
               end if
               j=j+1
               go to 30
            end if
            i=i+1
            go to 20
         end if
         go to 10
      end if
      return
      end
