      subroutine path(n,n3,seed,dffact,asrc,bsrc,sobj,atrgt,btrgt,
     +            c,nelt,maxelt,aelt,belt,celt,objb,objw,indb,
     +            indw,elitex,iter,iterb,ittype,
     +            adiff,bdiff,aprev,bprev,anext,bnext,
     +            alist,blist,backa,backb,backobj)   

 
      implicit none
c     ------------------------------------------------------------------
c     path: Path relinking between solution a,b obtained after the local
c     search and a randonly selected solution from the pool of elite
c     solutions.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     sobj    - object function value of source solution
c     tobj    - object function value of target solution
c     seed   - random number generator seed
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     objb   - cost of the best solution in the pool
c     objw   - cost of the worst solution in the pool
c     indb   - position of the best solution in the pool
c     indw   - position of the worst solution in the pool
c     elitex - flag that indicates if the pool of elite has changed
c     iterb  - iteration that found the  best solution 
c     iter   - number of GRASP iterations taken
c     ittype - type of iteration summary: = 0, silent
c                                         = 1, print improving solutions
c                                         = 2, print all iterations
c  
c     ------------------------------------------------------------------
      integer sobj,ittype,iter,iterb
      integer n,n3,seed,nelt,maxelt
      integer objb,objw,indb,indw
      real*4  dffact
      logical elitex
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c     - cost matrix
c     asrc  - array of source permutation a
c     bsrc  - array of source permutation b
c     atrgt - array of target permutation a 
c     atrgt - array of target permutation b
c     aelt  - a permutations array in the pool of elite solutions
c     belt  - b permutations array in the pool of elite solutions
c     celt  - array of elite solution costs
c     adiff  - array of symetric differences of a
c     aprev  - pointers to previous elements in differences list of a
c     anext  - pointers to next elements in differences list of a
c     alist  - differences linked list of a

c
c     bdiff  - array of symetric differences of b
c     bprev  - pointers to previous elements in differences list of b
c     bnext  - pointers to next elements in differences list ob b
c     blist  - differences linked list of b 
c     btrgt  - array of target permutation b in the pool
c
c     ------------------------------------------------------------------
      integer adiff(n),aprev(n),anext(n),alist(n)
      integer bdiff(n),bprev(n),bnext(n),blist(n) 
      integer backa(n),backb(n),backobj
      integer asrc(n),bsrc(n),atrgt(n),btrgt(n),c(n3)
      integer aelt(n*maxelt),belt(n*maxelt),celt(n*maxelt)
c     ------------------------------------------------------------------
c     Local scalars:
c     
c     afirst - first element in the linked list of a differences 
c     alast  - last  element in the linked list of a differences
c
c     bfirst - first element in the linked list of b differences
c     blast  - last  element in the linked list of b differences
c
c     sd     - function to calculate the simetric difference
c     delta  - distance between permutation arrays and targets
c     bgain  - best obtained gain
c     bmove  - index in the permutation  that gives the best gain
c     blmove - index in the list of moves that gives the best gain
c     abest - indicates if best move was obtained from a or b
c                if abest=true then a is regarded
c                if abest=false then b is regarded 
c     ------------------------------------------------------------------
      integer  afirst,alast,bfirst,blast 
      integer  delta,sd
      integer  bgain,bmove,blmove
      logical  abest
  
c     ------------------------------------------------------------------
c     Get symmetric difference arrays.
c     ------------------------------------------------------------------
      call savsol(n,asrc,bsrc,sobj,backa,backb,backobj)

      delta = sd(n,asrc,atrgt,adiff,aprev,anext,alist,afirst,alast)
                         
      delta = delta + sd(n,bsrc,btrgt,bdiff,
     +           bprev,bnext,blist,bfirst,blast)        

c     ------------------------------------------------------------------
c     Check if solution should be saved to be inserted in the pool.
c     ------------------------------------------------------------------
       
10    if (sobj.lt.backobj) 
     +    call savsol(n,asrc,bsrc,sobj,backa,backb,backobj)

      if(delta.gt.2) then

c     ------------------------------------------------------------------
c     Analyse all moves to get the one that has the lowest cost.
c     ------------------------------------------------------------------

         call pthmv(n,n3,c,asrc,bsrc,alist,adiff,anext,afirst,
     +      blist,bdiff,bnext,bfirst,abest,bgain,bmove,blmove)

c     ------------------------------------------------------------------
c     Update solution to best move.
c     ------------------------------------------------------------------

         sobj=sobj-bgain

         if (abest) then
             call updsd(n,atrgt,asrc,adiff,aprev,anext,
     +                alist,afirst,alast,bmove,blmove,delta)
          
         else  
             call updsd(n,btrgt,bsrc,bdiff,bprev,bnext,
     +               blist,bfirst,blast,bmove,blmove,delta) 

         endif
         goto  10
      endif

      call candpl(n,backa,backb,dffact,nelt,maxelt,aelt,belt,
     +           celt,objb,objw,indb,indw,backobj,
     +           elitex,iter,iterb,ittype)

      return
      end







