      subroutine path(n,n3,seed,dffact,obj,a,b,c,nelt,maxelt,
     /            aelt,belt,celt,objb,objw,indb,indw)   

      implicit none
c     ------------------------------------------------------------------
c     path: path relinking between solution a, b obtained after the local
c     search and a randonly selected solution from the pool of elite
c     solutions.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     obj    - object function value
c     seed   - random number generator seed
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     objb   - cost of the best solution in the pool
c     objw   - cost of the worst solution in the pool
c     indb   - position of the best solution in the pool
c     indw   - position of the worst solution in the pool
c     
c     ------------------------------------------------------------------
      integer n,n3,obj,seed,nelt,maxelt
      integer objb,objw,indb,indw
      real*4  dffact
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c     - cost matrix
c     a     - permutation array
c     b     - permutation array
c     aelt  - a permutations array in the pool of elite solutions
c     belt  - b permutations array in the pool of elite solutions
c     celt  - array of elite solution costs
c     
c     ------------------------------------------------------------------
      integer a(n),b(n),c(n3)
      integer aelt(n*maxelt),belt(n*maxelt),celt(n*maxelt)
c     ------------------------------------------------------------------
c     Local scalars:
c     
c     afirst - first element in the linked list of a differences 
c     alast  - last  element in the linked list of a differences
c
c     bfirst - first element in the linked list of b differences
c     blast  - last  element in the linked list of b differences
c
c     randp  - random number generator function
c     xrand  - dummy probability
c     ntrgt  - index of target permutation in the pool
c     sd     - function to calculate simetric difference
c     delta  - distance between permutation arrays and targets
c     bgain  - best obtained gain
c     bmove  - index in the permutation  that gives the best gain
c     blmove - index in the list of moves that gives the best gain
c     abest - indicates if best move was obtained from a or b
c                if abest=true then a is regarded
c                if abest=false then b is regarded 
c     ------------------------------------------------------------------
      integer  afirst,alast,bfirst,blast 
      real     randp,xrand
      integer  ntrgt, delta,sd
      integer  bgain,bmove,blmove
      logical  abest
c     ------------------------------------------------------------------
c     Local arrays:
c     adiff  - array of symetric differences of a
c     aprev  - pointers to previous elements in differences list of a
c     anext  - pointers to next elements in differences list of a
c     alist  - differences linked list of a
c
c     bdiff  - array of symetric differences of b
c     bprev  - pointers to previous elements in differences list of b
c     bnext  - pointers to next elements in differences list ob b
c     blist  - differences linked list of b 
c     ------------------------------------------------------------------
      integer adiff(n),aprev(n),anext(n),alist(n) 
      integer bdiff(n),bprev(n),bnext(n),blist(n)
c     ------------------------------------------------------------------
      integer iii
      if (nelt.eq.maxelt) then

         xrand=randp(seed)
         ntrgt=1+seed/(2147483647/nelt)
c         print *, 'Entrei ntrgt =',ntrgt 
      
         delta = sd(n,a,ntrgt,aelt,maxelt,adiff,aprev,
     \                    anext,alist,afirst,alast)

c         print *,'-------------------------------------------'
c         print *, 'delta =',delta
c         print *,'atarget =',(aelt(iii),iii=(ntrgt-1)*n+1,
c     \                (ntrgt-1)*n+n)
c         print *,'a =',(a(iii),iii=1,n) 
c         print *,'adiff =',(adiff(iii),iii=1,n) 
                      
c        print *,'Lista do A:' 
c              iii = afirst
c400           if(iii.ne.0) then
c                 print *, alist(iii)
c                 iii = anext(iii)
c                 goto 400
c          endif
c         call myflsh

         delta = delta + sd(n,b,ntrgt,belt,maxelt,bdiff,
     \                    bprev,bnext,blist,bfirst,blast)

c         print *,'     '
c         print *, 'delta =',delta
c         print *,'btarget =',(belt(iii),iii=(ntrgt-1)*n+1,
c     \                (ntrgt-1)*n+n)
c         print *,'b =',(b(iii),iii=1,n) 
c         print *,'bdiff =',(bdiff(iii),iii=1,n) 
            
c         print *,'Lista do B:' 
c         iii = bfirst
c300      if(iii.ne.0) then
c             print *, blist(iii)
c             iii = bnext(iii)
c             goto 300
c         endif


c         print *,'-------------------------------------------'
c         call myflsh



20      call candpl(n,a,b,dffact,nelt,maxelt,aelt,belt,
     \                 celt,objb,objw,indb,indw,obj)



        if(delta.gt.2) then

c     ------------------------------------------------------------------
c     Analyse all moves to get the one that has the lowest cost
c     ------------------------------------------------------------------

            call pthmv(n,n3,c,a,b,alist,adiff,anext,afirst,
     \            blist,bdiff,bnext,bfirst,abest,bgain,bmove,blmove)

c     ------------------------------------------------------------------
c     Update solution to best move
c     ------------------------------------------------------------------
           obj=obj-bgain

c           print *, '*** obj= ',obj
c           call myflsh

           if (abest) then
   
c              print *, 'Em A indice =',bmove,'custo=',bgain 
c              call myflsh
              call updsd(n,ntrgt,a,aelt,maxelt,adiff,aprev,anext,
     \                alist,afirst,alast,bmove,blmove,delta)

c              print *,'Lista do A:' 
c              iii = afirst
c100           if(iii.ne.0) then
c                print *, alist(iii)
c                 iii = anext(iii)
c                 goto 100
c              endif

c              print *, 'delta =',delta
c              print *,'atarget =',(aelt(iii),iii=(ntrgt-1)*n+1,
c     \                (ntrgt-1)*n+n)
c             print *,'a =',(a(iii),iii=1,n) 
c              print *,'adiff =',(adiff(iii),iii=1,n) 
c              call myflsh

          
           else  

            
c              print *, 'Em B indice =',bmove,'custo=',bgain    
              call updsd(n,ntrgt,b,belt,maxelt,bdiff,bprev,bnext,
     \               blist,bfirst,blast,bmove,blmove,delta) 

c              print *,'Lista do B:' 
c              iii = bfirst
c200           if(iii.ne.0) then
c                 print *, blist(iii)
c                 iii = bnext(iii)
c                 goto 200
c              endif         
c              print *,'     '
c              print *, 'delta =',delta
c              print *,'btarget =',(belt(iii),iii=(ntrgt-1)*n+1,
c     \                (ntrgt-1)*n+n)
c              print *,'b =',(b(iii),iii=1,n) 
c              print *,'bdiff =',(bdiff(iii),iii=1,n)  
c              call myflsh 

           endif
         
c           print *,'*************************************' 

          goto  20
        endif
      else
        print *,'Vou inserir nelt=',nelt
        call insrpl(n,a,b,nelt,maxelt,aelt,belt,
     \                   celt,objb,objw,indb,indw,obj)

        print *,'Inserido nelt=',nelt
        
      endif

      print *,'indb=',indb
      return
      end




