      subroutine pslcl  (n,n3,dffact,c,nelt,maxelt,
     +            aelt,belt,celt,objb,objw,indb,indw,
     +            elitex,iter,iterb,ittype)   

      implicit none
c     ------------------------------------------------------------------
c     pslocal: Applies local search to all solution in the pool.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     objb   - cost of the best solution in the pool
c     objw   - cost of the worst solution in the pool
c     indb   - position of the best solution in the pool
c     indw   - position of the worst solution in the pool
c     elitex - flag that indicates if the pool of elite has changed 
c     iterb  - iteration that found the  best solution 
c     iter   - number of GRASP iterations taken
c     ittype - type of iteration summary: = 0, silent
c                                         = 1, print improving solutions
c                                         = 2, print all iterations  
c  
c     ------------------------------------------------------------------
      integer n,n3,nelt,maxelt,ittype
      integer objb,objw,indb,indw,iter,iterb
      real*4  dffact
      logical elitex
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c     - cost matrix
c     aelt  - a permutations array in the pool of elite solutions
c     belt  - b permutations array in the pool of elite solutions
c     celt  - array of elite solution costs
c     
c     ------------------------------------------------------------------
      integer aelt(n*maxelt),belt(n*maxelt),celt(n*maxelt),c(n3)
c     ------------------------------------------------------------------

c     Local scalars:
c
c     nsrc   - index of source permutation in the pool
c     obj    - object function value
c     size  -  current size of the pool
c
c     ------------------------------------------------------------------ 
      integer nsrc,obj,size 
c     ------------------------------------------------------------------  
c     Local arrays:
c     a      - permutation array
c     b      - permutation array
c
c     ------------------------------------------------------------------
      integer a(n),b(n)
c     ------------------------------------------------------------------

      elitex =.false.
      size = nelt

      do 20 nsrc=1,size
        call getrgt(n,maxelt,nsrc,aelt,belt,celt,a,b,obj)

        call local(n,n3,obj,a,b,c)

        call  candpl(n,a,b,dffact,nelt,maxelt,aelt,belt,
     +             celt,objb,objw,indb,indw,obj,elitex,
     +             iter,iterb,ittype)
 
20    continue

      return
      end
