      subroutine psopt (n,n3,seedp,dffact,c,nelt,maxelt,
     +            aelt,belt,celt,objb,objw,
     +            indb,indw,iter,iterb,ittype,
     +            adiff,bdiff,aprev,bprev,anext,bnext,
     +            alist,blist,backa,backb,backobj)   

      implicit none
c     ------------------------------------------------------------------
c     psopt: Path relinking between  every pair of  elite solution
c     in the pool (in both directions).
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     obj    - object function value
c     seedp  - random number generator seed for path relinking
c     dffact - differentiation factor between a candidate solution
c                 and all solutions in the pool
c     nelt   - number of elite solutions in the pool
c     maxelt - maximum number of elite solutions in the pool
c     objb   - cost of the best solution in the pool
c     objw   - cost of the worst solution in the pool
c     indb   - position of the best solution in the pool
c     indw   - position of the worst solution in the pool
c     iterb  - iteration that found the  best solution 
c     iter   - number of GRASP iterations taken
c     ittype - type of iteration summary: = 0, silent
c                                         = 1, print improving solutions
c                                         = 2, print all iterations
c  
c     ------------------------------------------------------------------
      integer n,n3,obj,seedp,nelt,maxelt,ittype
      integer objb,objw,indb,indw,iter,iterb,backobj
      real*4  dffact
c     ------------------------------------------------------------------
c     Passed arrays:
c     
c     c     - cost matrix
c     aelt  - a permutations array in the pool of elite solutions
c     belt  - b permutations array in the pool of elite solutions
c     celt  - array of elite solution costs
c     adiff  - array of symetric differences of a
c     aprev  - pointers to previous elements in differences list of a
c     anext  - pointers to next elements in differences list of a
c     alist  - differences linked list of a
c     bdiff  - array of symetric differences of b
c     bprev  - pointers to previous elements in differences list of b
c     bnext  - pointers to next elements in differences list ob b
c     blist  - differences linked list of b 
c
c     ------------------------------------------------------------------
      integer adiff(n),aprev(n),anext(n),alist(n)
      integer bdiff(n),bprev(n),bnext(n),blist(n) 
      integer backa(n),backb(n)
      integer aelt(n*maxelt),belt(n*maxelt),celt(n*maxelt),c(n3)
c     ------------------------------------------------------------------
c     Local scalars:
c
c     nsrc   - index of permutation in the pool
c     ntrgt  - index of permutation in the pool
c     obj1   - object function value of permutation a1, b1
c     obj2   - object function value of permutation a2, b2
c     elitex - flag that indicates if the pool of elite has changed
c     objold - object function value before path relinking 
c              pos optimisation
c     size  -  current size of the pool
c
c     ------------------------------------------------------------------ 
      integer ntrgt,nsrc,size,obj1,obj2,objold
      logical elitex
c     ------------------------------------------------------------------  
c     Local arrays:
c     a1      - permutation array
c     b1      - permutation array
c     a2      - permutation array
c     b2      - permutation array
c
c     ------------------------------------------------------------------
      integer a1(n),b1(n),a2(n),b2(n)
c     ------------------------------------------------------------------
   
      elitex = .true.
10    if (.not.elitex) return

      elitex = .false.

      size = nelt

      do 30 nsrc=1,size
         do 20 ntrgt=1,size
           if (nsrc.ne.ntrgt) then 
              objold = objb 
              
              call getrgt(n,maxelt,nsrc,aelt,belt,celt,a1,b1,obj1)
              call getrgt(n,maxelt,ntrgt,aelt,belt,celt,a2,b2,obj2)

              call path(n,n3,seedp,dffact,a2,b2,obj2,a1,b1,
     +            c,nelt,maxelt,aelt,belt,celt,objb,objw,indb,
     +            indw,elitex,iter,iterb,ittype,
     +            adiff,bdiff,aprev,bprev,anext,bnext,
     +            alist,blist,backa,backb,backobj)
            
c            if ((ittype.eq.1).or.(ittype.eq.2).and.
c    +           (objb.lt.objold)) print *,'psopt: objb = ',objb
           endif
20       continue
30    continue
      goto 10

      end  
