      subroutine  pthmv(n,n3,c,a,b,alist,adiff,anext,afirst,
     +                 blist,bdiff,bnext,bfirst,abest,
     +                 bgain,bmove,blmove)
      implicit none
c     ------------------------------------------------------------------
c     pthmv:  Analyses all moves to get the one that has the
c     lowest cost.
c     ------------------------------------------------------------------
c     Passed scalars:
c     
c     n      - problem dimension
c     n3     - n * n * n
c     bgain  - best obtained gain
c     bmove  - index in the permutation  that gives the best gain
c     blmove - index in the list of moves that gives the best gain
c     afirst - first element in the linked list of differences of a
c     bfirst - first element in the linked list of differences of b
c     abest - indicates if best move was obtained from a or b
c                if abest=true then a is regarded
c                if abest=false then b is regarded 
c     ------------------------------------------------------------------
      integer n,n3
      integer bgain,bmove,blmove 
      integer afirst, bfirst
      logical abest
c     ------------------------------------------------------------------
c     Passed arrays:
c
c     c     - cost matrix
c     a     - permutation array
c     b     - permutation array
c     adiff - array of symetric differences of a
c     alist - differences linked list of a
c     anext - pointers to next elements in differences list of a
c     bdiff - array of symetric differences of b
c     blist - differences linked list of b 
c     bnext - pointers to next elements in differences list of b
c     ------------------------------------------------------------------
      integer a(n),b(n),c(n)
      integer adiff(n),alist(n),anext(n),bdiff(n),blist(n),bnext(n)
c     ------------------------------------------------------------------
c     Local scalars:
c
c     gain  - gain obtained by move
c     numel - number of considered element of the list
c     anotb - tells which permutation is updated 
c                if anotb=true then a is regarded
c                if anotb=false then b is regarded
c     lower - objective function improvement
c     i     -  element index
c     j     -  element index
c
c     ------------------------------------------------------------------
      integer gain, numel,i,j
      logical anotb, lower
c     ------------------------------------------------------------------
      
       numel = afirst  
       bgain = -2147483647

10     if (numel.ne.0) then
           anotb=.true.
           i = alist(numel)
           j = adiff(i)
           call costdf(n,n3,a,b,i,j,lower,c,anotb,gain) 
           if (gain.gt.bgain) then
               bgain = gain
               bmove  = i
               blmove  = numel
               abest = .true.
            endif
            numel = anext(numel)
            goto 10 
        endif

        numel = bfirst  

20      if (numel.ne.0) then
           anotb=.false.
           i = blist(numel)
           j = bdiff(i)
           call costdf(n,n3,a,b,i,j,lower,c,anotb,gain) 
           if (gain.gt.bgain) then
              bgain = gain
              bmove  = i
              blmove  = numel
              abest = .false.
            endif
            numel = bnext(numel)
            goto 20 
        endif
     
        return
        end


