       subroutine  updrcl(n,n3,srtic,ind,jind,kind,
     +               srnext,srprev,first, last,
     +               iprev,jprev,kprev,inext,jnext,knext,
     +               ifirst,ilast,jfirst,jlast,kfirst,
     +                klast,posind)

      implicit none

c     ------------------------------------------------------------
c     updrcl: Updates the restricted candidate list (RCL) 
c     -------------------------------------------------------------
c     Passed  Arrays:
c     
c     srtic   - sorted cost matrix (indices)
c     srnext  - next pointers to the sorted cost list
c     srprev  - previous pointers to the sorted cost list
c     inext   - next pointers to the list of valid i's
c     iprev   - previous pointers to the list of valid i's
c     jnext   - next pointers to the list of valid j's
c     jprev   - previous pointers to the list of valid j's
c     knext   - next pointers to the list of valid k's
c     kprev   - previous pointers to the list of valid k's
c     posind  - index of original array elements in the sorted  array
c
c     -------------------------------------------------------------   
      integer srtic(n3) 
      integer srnext(n3),srprev(n3) 
      integer inext(n), jnext(n), knext(n)
      integer iprev(n),jprev(n),kprev(n) 
      integer posind(n3)  
c     -------------------------------------------------------------
c     Passed  Scalars:
c     
c     n       - ap3 dimension
c     n3      - n * n * n
c     ind     - first index of assigned cost
c     jind    - second index of assigned cost
c     kind    - third index of assigned cost
c     first   - index in srtc of the first (minimum) permited cost value 
c     last    - index in srtc of the last  (maximum) permited cost value 
c     ifirst  - pointer to the first element of valid i's list
c     ilast   - pointer to the last element of valid i's list
c     jfirst  - pointer to the first element of valid j's list
c     jlast   - pointer to the last element of valid j's list
c     kfirst  - pointer to the first element of valid k's list
c     klast   - pointer to the last element of valid k's list
c
c     -------------------------------------------------------------
      integer n, n3
      integer ind, jind, kind
      integer first, last
      integer ifirst,ilast,jfirst,jlast,kfirst,klast
c     -------------------------------------------------------------
c     Local scalars:
c     
c     k      - loop index
c     i      - (first) index of cost
c     ai     - possible value of a(i)
c     bi     - possible value of b(i)
c     -------------------------------------------------------------
      integer curi, curj, curk    
c     -------------------------------------------------------------

c     ------------------------------------------------------------------
c     Remove from RCL all elements (i,j,k) s.t. i = ind chosen.
c     ------------------------------------------------------------------
   
c      print *,'indices:', ind,jind,kind

      call rmvsd(n,ind,iprev,inext,ifirst,ilast)
 
      curj = jfirst
10    if (curj.ne.0) then
       
        curk = kfirst
20      if (curk.ne.0) then
            call rmvsd(n3,posind(n*n*(ind-1)+n*(curj-1)+curk),
     +            srprev,srnext,first,last)
            curk = knext(curk)
            goto 20     
        endif

        curj = jnext(curj)
        goto 10     
      endif

c     ------------------------------------------------------------------
c     Remove from RCL all elements (i,j,k) s.t. j = jind chosen.
c     ------------------------------------------------------------------

      call rmvsd(n,jind,jprev,jnext,jfirst,jlast)

      curi = ifirst
30    if (curi.ne.0) then
       
        curk = kfirst
40      if (curk.ne.0) then
            call rmvsd(n3,posind(n*n*(curi-1)+n*(jind-1)+curk),
     +            srprev,srnext,first,last) 
            curk = knext(curk)
            goto 40     
        endif

        curi = inext(curi)
        goto 30     
      endif

c     ------------------------------------------------------------------
c     Remove from RCL all elements (i,j,k) s.t. k = kind chosen.
c     ------------------------------------------------------------------

      call rmvsd(n,kind,kprev,knext,kfirst,klast)

      curi = ifirst
50    if (curi.ne.0) then
       
        curj = jfirst
60      if (curj.ne.0) then
            call rmvsd(n3,posind(n*n*(curi-1)+n*(curj-1)+kind),
     +            srprev,srnext,first,last)

            curj = jnext(curj)
            goto 60     
        endif

        curi = inext(curi)
        goto 50     
      endif

      return
      end

